@@ -19,6 +19,7 @@
 #include <unistd.h>
 #include <fcntl.h>
 #include <sys/stat.h>
+#include <errno.h>
 
 #include "configuration.h"
 #include "display-manager.h"
@@ -146,6 +147,17 @@ display_manager_stopped_cb (DisplayManager *display_manager)
     g_main_loop_quit (loop);
 }
 
+static void
+display_manager_seat_removed_cb (DisplayManager *display_manager, Seat *seat)
+{
+    if (seat_get_boolean_property (seat, "exit-on-failure"))
+    {
+        g_debug ("Required seat has stopped");
+        exit_code = EXIT_FAILURE;
+        display_manager_stop (display_manager);
+    }
+}
+
 static GVariant *
 handle_display_manager_get_property (GDBusConnection       *connection,
                                      const gchar           *sender,
@@ -623,17 +635,7 @@ seat_added_cb (DisplayManager *display_manager, Seat *seat)
 static void
 seat_removed_cb (DisplayManager *display_manager, Seat *seat)
 {
-    gboolean exit_on_failure;
-
-    exit_on_failure = seat_get_boolean_property (seat, "exit-on-failure");
     g_hash_table_remove (seat_bus_entries, seat);
-
-    if (exit_on_failure)
-    {
-        g_debug ("Required seat has stopped");
-        exit_code = EXIT_FAILURE;
-        display_manager_stop (display_manager);
-    }
 }
 
 static void
@@ -1036,13 +1038,16 @@ main (int argc, char **argv)
 
     /* Create run and cache directories */
     dir = config_get_string (config_get_instance (), "LightDM", "log-directory");
-    g_mkdir_with_parents (dir, S_IRWXU | S_IXGRP | S_IXOTH);
+    if (g_mkdir_with_parents (dir, S_IRWXU | S_IXGRP | S_IXOTH) < 0)
+        g_warning ("Failed to make log directory %s: %s", dir, strerror (errno));
     g_free (dir);
     dir = config_get_string (config_get_instance (), "LightDM", "run-directory");
-    g_mkdir_with_parents (dir, S_IRWXU | S_IXGRP | S_IXOTH);
+    if (g_mkdir_with_parents (dir, S_IRWXU | S_IXGRP | S_IXOTH) < 0)
+        g_warning ("Failed to make run directory %s: %s", dir, strerror (errno));
     g_free (dir);
     dir = config_get_string (config_get_instance (), "LightDM", "cache-directory");
-    g_mkdir_with_parents (dir, S_IRWXU | S_IXGRP | S_IXOTH);
+    if (g_mkdir_with_parents (dir, S_IRWXU | S_IXGRP | S_IXOTH) < 0)
+        g_warning ("Failed to make cache directory %s: %s", dir, strerror (errno));
     g_free (dir);
 
     log_init ();
@@ -1069,6 +1074,7 @@ main (int argc, char **argv)
 
     display_manager = display_manager_new ();
     g_signal_connect (display_manager, "stopped", G_CALLBACK (display_manager_stopped_cb), NULL);
+    g_signal_connect (display_manager, "seat-removed", G_CALLBACK (display_manager_seat_removed_cb), NULL);
 
     /* Load the static display entries */
     groups = config_get_groups (config_get_instance ());
@@ -1118,7 +1124,10 @@ main (int argc, char **argv)
             g_object_unref (seat);
         }
         else
-            g_warning ("Failed to create default seat");
+        {
+            g_warning ("Failed to create default seat %s", type);
+            return EXIT_FAILURE;
+        }
     }
 
     display_manager_start (display_manager);
